#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <map>

using namespace std;

vector<int> color;
vector<pair<int, int>> timeout;
vector<bool> used;
vector<vector<int>> graph, backgraph, incolor, backincolor;
vector<int> indanger;
map<pair<int, int>, int> edges;
int n, m;
int timeline = 0;
const int MOD = 1e9 + 7;

void init() {
	graph.resize(n + 1);
	backgraph.resize(n + 1);
	incolor.resize(n + 1);
	backincolor.resize(n + 1);
	color.resize(n + 1, 0);
	timeout.resize(n + 1, {0, 0});
	for (int i = 1; i <= n; ++i)
		timeout[i].second = i;
	used.resize(n + 1, 0);
	indanger.resize(n + 1, m + 1);
}

void dfs(int u) {
	used[u] = 1;
	for (auto v : graph[u]) {
		if (!used[v])
			dfs(v);
	}
	timeout[u].first = ++timeline;
}

void colordfs(int u, int _color) {
	color[u] = _color;
	for(auto v : backgraph[u])
		if (!color[v]) {
			colordfs(v, _color);
		}
}

void dangerColor(int u) {
	int _color = color[u];
	if (incolor[u].size() == 0 || backincolor[u].size() == 0)
		return;
	if (incolor[u].size() == 1) {
		indanger[u] = min(indanger[u], edges[{u, incolor[u][0]}]);
	}
	queue<int> bfs;
	bfs.push(u);
	while (!bfs.empty()) {
		int x = bfs.front();
		bfs.pop();
		int dangergo = m + 1;
		int maxdangergo = 0;
		for (auto v : backincolor[x]) {
			dangergo = min(dangergo, indanger[x]);
			maxdangergo = max(maxdangergo, indanger[x]);
		}
		if (backincolor[x].size() == 1) {
			dangergo = edges[{backincolor[x][0], x}];
			maxdangergo = -1;
		}
		if (maxdangergo == m + 1)
			continue;
		indanger[x] = dangergo;
		for (auto v : incolor[x]) {
			if (indanger[v] > indanger[x]) {
				bfs.push(v);
			}
		}
	}
}

int ans(int a, int b) {
	if (a == b)
		return m + 1;
	if (color[a] != color[b])
		return 0;
	return min(indanger[a], indanger[b]);
}

void solve() {
	cin >> n >> m;
	init();
	for (int i = 0; i < m; ++i) {
		int u, v;
		cin >> u >> v;
		graph[u].push_back(v);
		backgraph[v].push_back(u);
		edges[{u, v}] = i + 1;
	}
	for (int i = 1; i <= n; ++i) {
		if (!used[i])
			dfs(i);
	}
	sort(timeout.rbegin(), timeout.rend());
	int curcolor = 0;
	for (int i = 0; i < n; ++i) {
		int u = timeout[i].second;
		if (!color[u])
			colordfs(u, ++curcolor);
	}
	for (int i = 1; i <= n; ++i) {
		for (auto v : graph[i]) {
			if (color[i] == color[v]) {
				incolor[i].push_back(v);
			}
		}
		for (auto v : backgraph[i]) {
			if (color[i] == color[v]) {
				backincolor[i].push_back(v);
			}
		}
	}
	for (int i = 1; i <= n; ++i) {
		dangerColor(i);
	}
	int q;
	cin >> q;
	long long BASE = 200000;
	long long res = 0;
	if (q == 0) {
		for (int i = 1; i <= n; ++i) {
			for (int j = i; j <= n; ++j) {
				res *= BASE;
				res += ans(i, j);
				res %= MOD;
			}
		}
	}
	else {
		while (q--) {
			int a, b;
			cin >> a >> b;
			res *= BASE;
			res += ans(a, b);
			res %= MOD;
		}
	}
	cout << res;
}

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	solve();
}