#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <map>

using namespace std;

vector<int> color;
vector<pair<int, int>> timeout;
vector<bool> used;
vector<vector<pair<int, int>>> graph, backgraph, incolor, backincolor;
vector<int> indanger;
vector<int> tempcolor;
int n, m;
int timeline = 0;
const int MOD = 1e9 + 7;

void init() {
	graph.resize(n + 1);
	backgraph.resize(n + 1);
	incolor.resize(n + 1);
	backincolor.resize(n + 1);
	color.resize(n + 1, 0);
	timeout.resize(n + 1, {0, 0});
	for (int i = 1; i <= n; ++i)
		timeout[i].second = i;
	used.resize(n + 1, 0);
	indanger.resize(n + 1, m + 1);
}

void dfs(int u) {
	used[u] = 1;
	for (auto [v, id] : graph[u]) {
		if (!used[v])
			dfs(v);
	}
	timeout[u].first = ++timeline;
}

void colordfs(int u, int _color) {
	color[u] = _color;
	for(auto [v, id] : backgraph[u])
		if (!color[v]) {
			colordfs(v, _color);
		}
}

int cant = 0;

void singledfs(int u) {
	used[u] = 1;
	for (auto [v, id] : graph[u]) {
		if (!used[v] && id != cant)
			singledfs(v);
	}
	timeout[u].first = ++timeline;
	timeout[u].second = u;
}

void singlecolordfs(int u, int _color) {
	tempcolor[u] = _color;
	for (auto [v, id] : backgraph[u])
		if (!tempcolor[v] && id != cant) {
			singlecolordfs(v, _color);
		}
}

void dangerColor(int u) {
	if (backincolor[u].size() != 1)
		return;
	cant = backincolor[u][0].second;
	timeline = 0;
	timeout.assign(n + 1, { 0, 0 });
	tempcolor.assign(n + 1, 0);
	used.assign(n + 1, 0);
	for (int i = 1; i <= n; ++i) {
		if (!used[u]) {
			singledfs(i);
		}
	}
	sort(timeout.rbegin(), timeout.rend());
	int curcolor = 0;
	for (auto [time, v] : timeout) {
		if (!tempcolor[v])
			singlecolordfs(v, ++curcolor);
	}
	for (int i = 1; i <= n; ++i) {
		if (tempcolor[i] == tempcolor[u]) {
			indanger[i] = min(indanger[i], cant);
		}
	}
}

void secdangerColor(int u) {
	if (incolor[u].size() != 1)
		return;
	cant = incolor[u][0].second;
	timeline = 0;
	timeout.assign(n + 1, { 0, 0 });
	tempcolor.assign(n + 1, 0);
	used.assign(n + 1, 0);
	for (int i = 1; i <= n; ++i) {
		if (!used[u]) {
			singledfs(i);
		}
	}
	sort(timeout.rbegin(), timeout.rend());
	int curcolor = 0;
	for (auto [time, v] : timeout) {
		if (!tempcolor[v])
			singlecolordfs(v, ++curcolor);
	}
	for (int i = 1; i <= n; ++i) {
		if (tempcolor[i] == tempcolor[u]) {
			indanger[i] = min(indanger[i], cant);
		}
	}
}

int ans(int a, int b) {
	if (a == b)
		return m + 1;
	if (color[a] != color[b])
		return 0;
	return min(indanger[a], indanger[b]);
}

void solve() {
	cin >> n >> m;
	init();
	for (int i = 0; i < m; ++i) {
		int u, v;
		cin >> u >> v;
		graph[u].push_back({ v, i + 1 });
		backgraph[v].push_back({ u, i + 1 });
	}
	for (int i = 1; i <= n; ++i) {
		if (!used[i])
			dfs(i);
	}
	sort(timeout.rbegin(), timeout.rend());
	int curcolor = 0;
	for (int i = 0; i < n; ++i) {
		int u = timeout[i].second;
		if (!color[u])
			colordfs(u, ++curcolor);
	}
	for (int i = 1; i <= n; ++i) {
		for (auto g : graph[i]) {
			if (color[i] == color[g.first]) {
				incolor[i].push_back(g);
			}
		}
		for (auto g : backgraph[i]) {
			if (color[i] == color[g.first]) {
				backincolor[i].push_back(g);
			}
		}
	}
	for (int i = 1; i <= n; ++i) {
		dangerColor(i);
	}
	for (int i = 1; i <= n; ++i) {
		secdangerColor(i);
	}
	int q;
	cin >> q;
	long long BASE = 200000;
	long long res = 0;
	if (q == 0) {
		for (int i = 1; i <= n; ++i) {
			for (int j = i; j <= n; ++j) {
				res *= BASE;
				res += ans(i, j);
				res %= MOD;
			}
		}
	}
	else {
		while (q--) {
			int a, b;
			cin >> a >> b;
			res *= BASE;
			res += ans(a, b);
			res %= MOD;
		}
	}
	cout << res;
}

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	solve();
}